@@ -2,6 +2,8 @@
 #define _EET_H
 
 #include <stdlib.h>
+#include <stdio.h>
+#include <Eina.h>
 
 #ifdef EAPI
 # undef EAPI
@@ -56,7 +58,8 @@ extern "C" {
 #define EET_T_ULONG_LONG        10 /**< Data type: unsigned long long */
 #define EET_T_STRING            11 /**< Data type: char * */
 #define EET_T_INLINED_STRING    12 /**< Data type: char * (but compressed inside the resulting eet) */
-#define EET_T_LAST              13 /**< Last data type */
+#define EET_T_NULL              13 /**< Data type: (void *) (only use it if you know why) */
+#define EET_T_LAST              14 /**< Last data type */
 
 #define EET_G_UNKNOWN    100 /**< Unknown group data encoding type */
 #define EET_G_ARRAY      101 /**< Fixed size array group type */
@@ -86,12 +89,23 @@ extern "C" {
 	EET_ERROR_WRITE_ERROR_FILE_TOO_BIG,
 	EET_ERROR_WRITE_ERROR_IO_ERROR,
 	EET_ERROR_WRITE_ERROR_OUT_OF_SPACE,
-	EET_ERROR_WRITE_ERROR_FILE_CLOSED
+	EET_ERROR_WRITE_ERROR_FILE_CLOSED,
+	EET_ERROR_MMAP_FAILED,
+	EET_ERROR_X509_ENCODING_FAILED,
+	EET_ERROR_SIGNATURE_FAILED,
+	EET_ERROR_INVALID_SIGNATURE,
+	EET_ERROR_NOT_SIGNED,
+	EET_ERROR_NOT_IMPLEMENTED,
+	EET_ERROR_PRNG_NOT_SEEDED,
+	EET_ERROR_ENCRYPT_FAILED,
+	EET_ERROR_DECRYPT_FAILED
      } Eet_Error;
 
    typedef struct _Eet_File                  Eet_File;
    typedef struct _Eet_Dictionary            Eet_Dictionary;
    typedef struct _Eet_Data_Descriptor       Eet_Data_Descriptor;
+   typedef struct _Eet_Key                   Eet_Key;
+   typedef struct _Eet_Node                  Eet_Node;
 
    typedef struct _Eet_Data_Descriptor_Class Eet_Data_Descriptor_Class;
 
@@ -214,6 +228,16 @@ extern "C" {
     */
    EAPI Eet_File *eet_open(const char *file, Eet_File_Mode mode);
 
+   /**
+    * Open an eet file directly from a memory location. The data are not copied,
+    * so you must keep them around as long as the eet file is open. Their is
+    * currently no cache for this kind of Eet_File, so it's reopen every time
+    * you do use eet_memopen_read.
+    *
+    * @since 2.0.0
+    */
+   EAPI Eet_File *eet_memopen_read(const void *data, size_t size);
+
    /**
     * Get the mode an Eet_File was opened with.
     * @param ef A valid eet file handle.
@@ -237,6 +261,74 @@ extern "C" {
     */
    EAPI Eet_Error eet_close(Eet_File *ef);
 
+  /**
+   * Callback used to request if needed the password of a private key.
+   *
+   * @since 2.0.0
+   */
+   typedef int (*Eet_Key_Password_Callback)(char *buffer, int size, int rwflag, void *data);
+
+   /**
+    * Create an Eet_Key needed for signing an eet file.
+    *
+    * The certificate should provide the public that match the private key.
+    * No verification is done to ensure that.
+    *
+    * @since 2.0.0
+    */
+   EAPI Eet_Key* eet_identity_open(const char *certificate_file, const char *private_key_file, Eet_Key_Password_Callback cb);
+
+    /**
+     * Close and release all ressource used by an Eet_Key.
+     * An reference counter prevent it from being freed until all file using it are
+     * also closed.
+     *
+     * @since 2.0.0
+     */
+   EAPI void eet_identity_close(Eet_Key *key);
+
+    /**
+     * Set a key to sign a file
+     *
+     * @since 2.0.0
+     */
+   EAPI Eet_Error eet_identity_set(Eet_File *ef, Eet_Key *key);
+
+    /**
+     * Display both private and public key of an Eet_Key.
+     *
+     * @since 2.0.0
+     */
+   EAPI void eet_identity_print(Eet_Key *key, FILE *out);
+
+    /**
+     * Get the x509 der certificate associated with an Eet_File. Will return NULL
+     * if the file is not signed.
+     *
+     * @since 2.0.0
+     */
+   EAPI const void *eet_identity_x509(Eet_File *ef, int *der_length);
+
+    /**
+     * Get the raw signature associated with an Eet_File. Will return NULL
+     * if the file is not signed.
+     */
+   EAPI const void *eet_identity_signature(Eet_File *ef, int *signature_length);
+
+    /**
+     * Get the SHA1 associated with a file. Could be the one used to sign the data
+     * or if the data where not signed, it will be the SHA1 of the file.
+     *
+     * @since 2.0.0
+     */
+   EAPI const void *eet_identity_sha1(Eet_File *ef, int *sha1_length);
+
+   /**
+    * Display the x509 der certificate to out.
+    *
+    * @since 2.0.0
+    */
+   EAPI void eet_identity_certificate_print(const unsigned char *certificate, int der_length, FILE *out);
 
    /**
     * Return a handle to the shared string dictionary of the Eet file
@@ -285,6 +377,7 @@ extern "C" {
     *
     * @since 1.0.0
     */
+   EAPI void *eet_read_cipher(Eet_File *ef, const char *name, int *size_ret, const char *cipher_key);
    EAPI void *eet_read(Eet_File *ef, const char *name, int *size_ret);
 
    /**
@@ -333,6 +426,7 @@ extern "C" {
     *
     * @since 1.0.0
     */
+   EAPI int eet_write_cipher(Eet_File *ef, const char *name, const void *data, int size, int compress, const char *cipher_key);
    EAPI int eet_write(Eet_File *ef, const char *name, const void *data, int size, int compress);
 
    /**
@@ -430,6 +524,7 @@ extern "C" {
     *
     * @since 1.0.0
     */
+   EAPI int eet_data_image_header_read_cipher(Eet_File *ef, const char *name, const char *key, unsigned int *w, unsigned int *h, int *alpha, int *compress, int *quality, int *lossy);
    EAPI int eet_data_image_header_read(Eet_File *ef, const char *name, unsigned int *w, unsigned int *h, int *alpha, int *compress, int *quality, int *lossy);
 
    /**
@@ -467,8 +562,49 @@ extern "C" {
     *
     * @since 1.0.0
     */
+   EAPI void *eet_data_image_read_cipher(Eet_File *ef, const char *name, const char *key, unsigned int *w, unsigned int *h, int *alpha, int *compress, int *quality, int *lossy);
    EAPI void *eet_data_image_read(Eet_File *ef, const char *name, unsigned int *w, unsigned int *h, int *alpha, int *compress, int *quality, int *lossy);
 
+   /**
+    * Read image data from the named key in the eet file.
+    * @param ef A valid eet file handle opened for reading.
+    * @param name Name of the entry. eg: "/base/file_i_want".
+    * @param src_x The starting x coordinate from where to dump the stream.
+    * @param src_y The starting y coordinate from where to dump the stream.
+    * @param d A pointer to the pixel surface.
+    * @param w The expected width in pixels of the pixel surface to decode.
+    * @param h The expected height in pixels of the pixel surface to decode.
+    * @param row_stride The length of a pixels line in the destination surface.
+    * @param alpha A pointer to the int to hold the alpha flag.
+    * @param compress A pointer to the int to hold the compression amount.
+    * @param quality A pointer to the int to hold the quality amount.
+    * @param lossy A pointer to the int to hold the lossiness flag.
+    * @return 1 on success, 0 otherwise.
+    *
+    * This function reads an image from an eet file stored under the named
+    * key in the eet file and return a pointer to the decompressed pixel data.
+    *
+    * The other parameters of the image (width, height etc.) are placed into
+    * the values pointed to (they must be supplied). The pixel data is a linear
+    * array of pixels starting from the top-left of the image scanning row by
+    * row from left to right. Each pile is a 32bit value, with the high byte
+    * being the alpha channel, the next being red, then green, and the low byte
+    * being blue. The width and height are measured in pixels and will be
+    * greater than 0 when returned. The alpha flag is either 0 or 1. 0 denotes
+    * that the alpha channel is not used. 1 denotes that it is significant.
+    * Compress is filled with the compression value/amount the image was
+    * stored with. The quality value is filled with the quality encoding of
+    * the image file (0 - 100). The lossy flags is either 0 or 1 as to if
+    * the image was encoded lossily or not.
+    *
+    * On success the function returns 1, and 0 on failure. On failure the
+    * parameter values may not contain any sensible data.
+    *
+    * @since 1.0.2
+    */
+   EAPI int eet_data_image_read_to_surface_cipher(Eet_File *ef, const char *name, const char *key, unsigned int src_x, unsigned int src_y, unsigned int *d, unsigned int w, unsigned int h, unsigned int row_stride, int *alpha, int *compress, int *quality, int *lossy);
+   EAPI int eet_data_image_read_to_surface(Eet_File *ef, const char *name, unsigned int src_x, unsigned int src_y, unsigned int *d, unsigned int w, unsigned int h, unsigned int row_stride, int *alpha, int *compress, int *quality, int *lossy);
+
    /**
     * Write image data to the named key in an eet file.
     * @param ef A valid eet file handle opened for writing.
@@ -501,6 +637,7 @@ extern "C" {
     *
     * @since 1.0.0
     */
+   EAPI int eet_data_image_write_cipher(Eet_File *ef, const char *name, const char *key, const void *data, unsigned int w, unsigned int h, int alpha, int compress, int quality, int lossy);
    EAPI int eet_data_image_write(Eet_File *ef, const char *name, const void *data, unsigned int w, unsigned int h, int alpha, int compress, int quality, int lossy);
 
    /**
@@ -536,6 +673,7 @@ extern "C" {
     *
     * @since 1.0.0
     */
+   EAPI int eet_data_image_header_decode_cipher(const void *data, const char *key, int size, unsigned int *w, unsigned int *h, int *alpha, int *compress, int *quality, int *lossy);
    EAPI int eet_data_image_header_decode(const void *data, int size, unsigned int *w, unsigned int *h, int *alpha, int *compress, int *quality, int *lossy);
 
    /**
@@ -573,8 +711,49 @@ extern "C" {
     *
     * @since 1.0.0
     */
+   EAPI void *eet_data_image_decode_cipher(const void *data, const char *key, int size, unsigned int *w, unsigned int *h, int *alpha, int *compress, int *quality, int *lossy);
    EAPI void *eet_data_image_decode(const void *data, int size, unsigned int *w, unsigned int *h, int *alpha, int *compress, int *quality, int *lossy);
 
+   /**
+    * Decode Image data into pixel data.
+    * @param data The encoded pixel data.
+    * @param size The size, in bytes, of the encoded pixel data.
+    * @param src_x The starting x coordinate from where to dump the stream.
+    * @param src_y The starting y coordinate from where to dump the stream.
+    * @param d A pointer to the pixel surface.
+    * @param w The expected width in pixels of the pixel surface to decode.
+    * @param h The expected height in pixels of the pixel surface to decode.
+    * @param row_stride The length of a pixels line in the destination surface.
+    * @param alpha A pointer to the int to hold the alpha flag.
+    * @param compress A pointer to the int to hold the compression amount.
+    * @param quality A pointer to the int to hold the quality amount.
+    * @param lossy A pointer to the int to hold the lossiness flag.
+    * @return 1 on success, 0 otherwise.
+    *
+    * This function takes encoded pixel data and decodes it into raw RGBA
+    * pixels on success.
+    *
+    * The other parameters of the image (alpha, compress etc.) are placed into
+    * the values pointed to (they must be supplied). The pixel data is a linear
+    * array of pixels starting from the top-left of the image scanning row by
+    * row from left to right. Each pixel is a 32bit value, with the high byte
+    * being the alpha channel, the next being red, then green, and the low byte
+    * being blue. The width and height are measured in pixels and will be
+    * greater than 0 when returned. The alpha flag is either 0 or 1. 0 denotes
+    * that the alpha channel is not used. 1 denotes that it is significant.
+    * Compress is filled with the compression value/amount the image was
+    * stored with. The quality value is filled with the quality encoding of
+    * the image file (0 - 100). The lossy flags is either 0 or 1 as to if
+    * the image was encoded lossily or not.
+    *
+    * On success the function returns 1, and 0 on failure. On failure the
+    * parameter values may not contain any sensible data.
+    *
+    * @since 1.0.2
+    */
+   EAPI int eet_data_image_decode_to_surface_cipher(const void *data, const char *key, int size, unsigned int src_x, unsigned int src_y, unsigned int *d, unsigned int w, unsigned int h, unsigned int row_stride, int *alpha, int *compress, int *quality, int *lossy);
+   EAPI int eet_data_image_decode_to_surface(const void *data, int size, unsigned int src_x, unsigned int src_y, unsigned int *d, unsigned int w, unsigned int h, unsigned int row_stride, int *alpha, int *compress, int *quality, int *lossy);
+
    /**
     * Encode image data for storage or transmission.
     * @param data A pointer to the image pixel data.
@@ -606,6 +785,7 @@ extern "C" {
     *
     * @since 1.0.0
     */
+   EAPI void *eet_data_image_encode_cipher(const void *data, const char *key, unsigned int w, unsigned int h, int alpha, int compress, int quality, int lossy, int *size_ret);
    EAPI void *eet_data_image_encode(const void *data, int *size_ret, unsigned int w, unsigned int h, int alpha, int compress, int quality, int lossy);
 
 /***************************************************************************/
@@ -668,10 +848,20 @@ extern "C" {
     *    double floating_lots;
     *    char *string;
     *    Blah2 *blah2;
-    *    Evas_List *blah3;
+    *    Eina_List *blah3;
     * }
     * Blah;
     *
+    * Eina_Hash*
+    * eet_eina_hash_add(Eina_Hash *hash, const char *key, const void *data)
+    * {
+    *    if (!hash) hash = eina_hash_string_superfast_new(NULL);
+    *    if (!hash) return NULL;
+    *
+    *    eina_hash_add(hash, key, data);
+    *    return hash;
+    * }
+    *
     * int
     * main(int argc, char **argv)
     * {
@@ -685,33 +875,33 @@ extern "C" {
     *    Blah *blah_in;
     *
     *    edd3 = eet_data_descriptor_new("blah3", sizeof(Blah3),
-    *                                   evas_list_next,
-    *                                   evas_list_append,
-    *                                   evas_list_data,
-    *                                   evas_list_free,
-    *                                   evas_hash_foreach,
-    *                                   evas_hash_add,
-    *                                   evas_hash_free);
+    *                                   eina_list_next,
+    *                                   eina_list_append,
+    *                                   eina_list_data_get,
+    *                                   eina_list_free,
+    *                                   eina_hash_foreach,
+    *                                   eet_eina_hash_add,
+    *                                   eina_hash_free);
     *    EET_DATA_DESCRIPTOR_ADD_BASIC(edd3, Blah3, "string3", string, EET_T_STRING);
     *
     *    edd2 = eet_data_descriptor_new("blah2", sizeof(Blah2),
-    *                                   evas_list_next,
-    *                                   evas_list_append,
-    *                                   evas_list_data,
-    *                                   evas_list_free,
-    *                                   evas_hash_foreach,
-    *                                   evas_hash_add,
-    *                                   evas_hash_free);
+    *                                   eina_list_next,
+    *                                   eina_list_append,
+    *                                   eina_list_data_get,
+    *                                   eina_list_free,
+    *                                   eina_hash_foreach,
+    *                                   eet_eina_hash_add,
+    *                                   eina_hash_free);
     *    EET_DATA_DESCRIPTOR_ADD_BASIC(edd2, Blah2, "string2", string, EET_T_STRING);
     *
     *    edd = eet_data_descriptor_new("blah", sizeof(Blah),
-    *                                   evas_list_next,
-    *                                   evas_list_append,
-    *                                   evas_list_data,
-    *                                   evas_list_free,
-    *                                   evas_hash_foreach,
-    *                                   evas_hash_add,
-    *                                   evas_hash_free);
+    *                                   eina_list_next,
+    *                                   eina_list_append,
+    *                                   eina_list_data_get,
+    *                                   eina_list_free,
+    *                                   eina_hash_foreach,
+    *                                   eet_eina_hash_add,
+    *                                   eina_hash_free);
     *    EET_DATA_DESCRIPTOR_ADD_BASIC(edd, Blah, "character", character, EET_T_CHAR);
     *    EET_DATA_DESCRIPTOR_ADD_BASIC(edd, Blah, "sixteen", sixteen, EET_T_SHORT);
     *    EET_DATA_DESCRIPTOR_ADD_BASIC(edd, Blah, "integer", integer, EET_T_INT);
@@ -734,13 +924,13 @@ extern "C" {
     *    blah.floating_lots=0.777777777777777;
     *    blah.string="bite me like a turnip";
     *    blah.blah2 = &blah2;
-    *    blah.blah3 = evas_list_append(NULL, &blah3);
-    *    blah.blah3 = evas_list_append(blah.blah3, &blah3);
-    *    blah.blah3 = evas_list_append(blah.blah3, &blah3);
-    *    blah.blah3 = evas_list_append(blah.blah3, &blah3);
-    *    blah.blah3 = evas_list_append(blah.blah3, &blah3);
-    *    blah.blah3 = evas_list_append(blah.blah3, &blah3);
-    *    blah.blah3 = evas_list_append(blah.blah3, &blah3);
+    *    blah.blah3 = eina_list_append(NULL, &blah3);
+    *    blah.blah3 = eina_list_append(blah.blah3, &blah3);
+    *    blah.blah3 = eina_list_append(blah.blah3, &blah3);
+    *    blah.blah3 = eina_list_append(blah.blah3, &blah3);
+    *    blah.blah3 = eina_list_append(blah.blah3, &blah3);
+    *    blah.blah3 = eina_list_append(blah.blah3, &blah3);
+    *    blah.blah3 = eina_list_append(blah.blah3, &blah3);
     *
     *    data = eet_data_descriptor_encode(edd, &blah, &size);
     *    printf("-----DECODING\n");
@@ -757,13 +947,11 @@ extern "C" {
     *    printf("%p\n", blah_in->blah2);
     *    printf("  %s\n", blah_in->blah2->string);
     *      {
-    *         Evas_List *l;
+    *         Eina_List *l;
+    *         Blah3 *blah3_in;
     *
-    *         for (l = blah_in->blah3; l; l = l->next)
+    *         EINA_LIST_FOREACH(blah_in->blah3, l, blah3_in)
     *           {
-    *              Blah3 *blah3_in;
-    *
-    *              blah3_in = l->data;
     *              printf("%p\n", blah3_in);
     *              printf("  %s\n", blah3_in->string);
     *           }
@@ -811,7 +999,7 @@ extern "C" {
     *
     * @since 1.0.0
     */
-   EAPI void eet_data_descriptor_element_add(Eet_Data_Descriptor *edd, const char *name, int type, int group_type, int offset, int count, const char *counter_name, Eet_Data_Descriptor *subtype);
+   EAPI void eet_data_descriptor_element_add(Eet_Data_Descriptor *edd, const char *name, int type, int group_type, int offset, /* int count_offset,  */int count, const char *counter_name, Eet_Data_Descriptor *subtype);
 
    /**
     * Read a data structure from an eet file and decodes it.
@@ -837,6 +1025,7 @@ extern "C" {
     *
     * @since 1.0.0
     */
+   EAPI void *eet_data_read_cipher(Eet_File *ef, Eet_Data_Descriptor *edd, const char *name, const char *key);
    EAPI void *eet_data_read(Eet_File *ef, Eet_Data_Descriptor *edd, const char *name);
 
    /**
@@ -853,6 +1042,7 @@ extern "C" {
     *
     * @since 1.0.0
     */
+   EAPI int eet_data_write_cipher(Eet_File *ef, Eet_Data_Descriptor *edd, const char *name, const char *key, const void *data, int compress);
    EAPI int eet_data_write(Eet_File *ef, Eet_Data_Descriptor *edd, const char *name, const void *data, int compress);
 
    /**
@@ -898,6 +1088,7 @@ extern "C" {
     *
     * @since 1.0.0
     */
+   EAPI int eet_data_text_dump_cipher(const void *data_in, const char *key, int size_in, void (*dumpfunc) (void *data, const char *str), void *dumpdata);
    EAPI int eet_data_text_dump(const void *data_in, int size_in, void (*dumpfunc) (void *data, const char *str), void *dumpdata);
 
    /**
@@ -914,6 +1105,7 @@ extern "C" {
     *
     * @since 1.0.0
     */
+   EAPI void *eet_data_text_undump_cipher(const char *text, const char *key, int textlen, int *size_ret);
    EAPI void *eet_data_text_undump(const char *text, int textlen, int *size_ret);
 
    /**
@@ -933,6 +1125,7 @@ extern "C" {
     *
     * @since 1.0.0
     */
+   EAPI int eet_data_dump_cipher(Eet_File *ef, const char *name, const char *key, void (*dumpfunc) (void *data, const char *str), void *dumpdata);
    EAPI int eet_data_dump(Eet_File *ef, const char *name, void (*dumpfunc) (void *data, const char *str), void *dumpdata);
 
    /**
@@ -953,6 +1146,7 @@ extern "C" {
     *
     * @since 1.0.0
     */
+   EAPI int eet_data_undump_cipher(Eet_File *ef, const char *name, const char *key, const char *text, int textlen, int compress);
    EAPI int eet_data_undump(Eet_File *ef, const char *name, const char *text, int textlen, int compress);
 
    /**
@@ -979,6 +1173,7 @@ extern "C" {
     *
     * @since 1.0.0
     */
+   EAPI void *eet_data_descriptor_decode_cipher(Eet_Data_Descriptor *edd, const void *data_in, const char *key, int size_in);
    EAPI void *eet_data_descriptor_decode(Eet_Data_Descriptor *edd, const void *data_in, int size_in);
 
    /**
@@ -1007,6 +1202,7 @@ extern "C" {
     *
     * @since 1.0.0
     */
+   EAPI void *eet_data_descriptor_encode_cipher(Eet_Data_Descriptor *edd, const void *data_in, const char *key, int *size_ret);
    EAPI void *eet_data_descriptor_encode(Eet_Data_Descriptor *edd, const void *data_in, int *size_ret);
 
    /**
@@ -1036,7 +1232,7 @@ eet_dictionary_string_check    * example: values), and @p type is the basic data
 	\
 	eet_data_descriptor_element_add(edd, name, type, EET_G_UNKNOWN, \
 					(char *)(&(___ett.member)) - (char *)(&(___ett)), \
-					0, NULL, NULL); \
+					0, /* 0,  */NULL, NULL); \
      }
 
    /**
@@ -1061,7 +1257,7 @@ eet_dictionary_string_check    * example: values), and @p type is the basic data
 	\
 	eet_data_descriptor_element_add(edd, name, EET_T_UNKNOW, EET_G_UNKNOWN, \
 					(char *)(&(___ett.member)) - (char *)(&(___ett)), \
-					0, NULL, subtype); \
+					0, /* 0,  */NULL, subtype); \
      }
 
    /**
@@ -1085,7 +1281,7 @@ eet_dictionary_string_check    * example: values), and @p type is the basic data
 	\
 	eet_data_descriptor_element_add(edd, name, EET_T_UNKNOW, EET_G_LIST, \
 					(char *)(&(___ett.member)) - (char *)(&(___ett)), \
-					0, NULL, subtype); \
+					0, /* 0,  */NULL, subtype); \
      }
 
    /**
@@ -1109,10 +1305,84 @@ eet_dictionary_string_check    * example: values), and @p type is the basic data
 	\
 	eet_data_descriptor_element_add(edd, name, EET_T_UNKNOW, EET_G_HASH, \
 					(char *)(&(___ett.member)) - (char *)(&(___ett)), \
-					0, NULL, subtype); \
+					0, /* 0,  */NULL, subtype); \
+     }
+
+   /**
+    * Add a fixed size array type to a data descriptor
+    * @param edd The data descriptor to add the type to.
+    * @param struct_type The type of the struct.
+    * @param name The string name to use to encode/decode this member (must be a constant global and never change).
+    * @param member The struct member itself to be encoded.
+    * @param subtype The type of hash member to add.
+    *
+    * This macro lets you easily add a fixed size array of other data types. All the
+    * parameters are the same as for EET_DATA_DESCRIPTOR_ADD_BASIC(), with the
+    * @p subtype being the exception. This must be the data descriptor of the
+    * element that is in each member of the hash to be stored.
+    *
+    * @since 1.0.2
+    */
+#define EET_DATA_DESCRIPTOR_ADD_ARRAY(edd, struct_type, name, member, subtype) \
+     { \
+	struct_type ___ett; \
+	\
+	eet_data_descriptor_element_add(edd, name, EET_T_UNKNOW, EET_G_ARRAY, \
+					(char *)(&(___ett.member)) - (char *)(&(___ett)), \
+					/* 0,  */sizeof(___ett.member)/sizeof(___ett.member[0]), NULL, subtype); \
+     }
+
+   /**
+    * Add a variable size array type to a data descriptor
+    * @param edd The data descriptor to add the type to.
+    * @param struct_type The type of the struct.
+    * @param name The string name to use to encode/decode this member (must be a constant global and never change).
+    * @param member The struct member itself to be encoded.
+    * @param subtype The type of hash member to add.
+    *
+    * This macro lets you easily add a fixed size array of other data types. All the
+    * parameters are the same as for EET_DATA_DESCRIPTOR_ADD_BASIC(), with the
+    * @p subtype being the exception. This must be the data descriptor of the
+    * element that is in each member of the hash to be stored.
+    *
+    * @since 1.0.2
+    */
+#define EET_DATA_DESCRIPTOR_ADD_VAR_ARRAY(edd, struct_type, name, member, subtype) \
+     { \
+	struct_type ___ett; \
+	\
+	eet_data_descriptor_element_add(edd, name, EET_T_UNKNOW, EET_G_VAR_ARRAY, \
+					(char *)(&(___ett.member)) - (char *)(&(___ett)), \
+					(char *)(&(___ett.member ## _count)) - (char *)(&(___ett)), /* 0,  */NULL, subtype); \
      }
 
 /***************************************************************************/
+
+   EAPI Eet_Node *eet_node_char_new(const char *name, char c);
+   EAPI Eet_Node *eet_node_short_new(const char *name, short s);
+   EAPI Eet_Node *eet_node_int_new(const char *name, int i);
+   EAPI Eet_Node *eet_node_long_long_new(const char *name, long long l);
+   EAPI Eet_Node *eet_node_float_new(const char *name, float f);
+   EAPI Eet_Node *eet_node_double_new(const char *name, double d);
+   EAPI Eet_Node *eet_node_unsigned_char_new(const char *name, unsigned char uc);
+   EAPI Eet_Node *eet_node_unsigned_short_new(const char *name, unsigned short us);
+   EAPI Eet_Node *eet_node_unsigned_int_new(const char *name, unsigned int ui);
+   EAPI Eet_Node *eet_node_unsigned_long_long_new(const char *name, unsigned long long l);
+   EAPI Eet_Node *eet_node_string_new(const char *name, const char *str);
+   EAPI Eet_Node *eet_node_inlined_string_new(const char *name, const char *str);
+   EAPI Eet_Node *eet_node_null_new(const char *name);
+   EAPI Eet_Node *eet_node_list_new(const char *name, Eina_List *nodes);
+   EAPI Eet_Node *eet_node_array_new(const char *name, int count, Eina_List *nodes);
+   EAPI Eet_Node *eet_node_var_array_new(const char *name, Eina_List *nodes);
+   EAPI Eet_Node *eet_node_hash_new(const char *name, const char *key, Eet_Node *node);
+   EAPI Eet_Node *eet_node_struct_new(const char *name, Eina_List *nodes);
+   EAPI void eet_node_del(Eet_Node *n);
+
+   EAPI void *eet_data_node_encode_cipher(Eet_Node *node, const char *key, int *size_ret);
+   EAPI int eet_data_node_write_cipher(Eet_File *ef, const char *name, const char *key, Eet_Node *node, int compress);
+
+/***************************************************************************/
+
 #ifdef __cplusplus
 }
 #endif
